#!/bin/sh
#
# mbuild - automate builds with sbuild and dinstall the result
#
# Copyright (c) 2008 martin f. krafft <madduck@debian.org>
# Released under the terms of the Artistic Licence 2.0.
#
set -eu

BUILDDIR="$(mktemp -dt mbuild.XXXXXXXX)"
trap "cd / && rm -R '$BUILDDIR'" 0

BUNDLE_DIR="${TMPDIR:-/tmp}"
dinstall() {
  echo "Would install into $1:"
  for file in $(dcmd echo "$2"); do
    echo "  $file"
  done
}

KEYID=

MBUILDRCS="/etc/mbuild/rc $HOME/.mbuildrc $HOME/.mbuild/rc"
for rc in $MBUILDRCS; do
  [ -r "$rc" ] && . "$rc"
done

if [ -z "$KEYID" ]; then
  echo "E: \$KEYID is not defined in rc file." >&2
  exit 1
fi

cd "$BUILDDIR"

sbuild_args=
schangesfile=
arch=
dist=
for opt in "$@"; do
  case "$opt" in
    --arch=*) arch="${opt#--arch=}";;
    --dist=*) dist="${opt#--dist=}";;
    -sa) sbuild_args="${sbuild_args:+$sbuild_args }--force-orig-source";;
    --*) sbuild_args="${sbuild_args:+$sbuild_args }$opt";;
    *_source.changes)
      if [ -z "$schangesfile" ]; then
        if [ -f "$opt" ] && [ -r "$opt" ]; then
          schangesfile="$opt"
        else
          echo "E: file does not exist: $opt" >&2
          exit 1
        fi
      else
        echo "E: cannot pass more than one source changes file: $opt" >&2
        exit 1
      fi
      ;;
    *)
      echo "E: unknown option: $opt" >&1
      exit 1
      ;;
  esac
done

schangesfile_basename="${schangesfile##*/}"
packagenameversion="${schangesfile_basename%_*}"

cd "${schangesfile%/*}"
dcmd cp "${schangesfile##*/}" "$BUILDDIR"
schangesfile="$schangesfile_basename"

cd "$BUILDDIR"

sbuild $sbuild_args --arch="$arch" --arch-all --dist="$dist" --keyid=$KEYID \
  "$packagenameversion".dsc

changesfile="${packagenameversion}_${arch}.changes"

dinstall "$dist" "$schangesfile"
dinstall "$dist" "$changesfile"

mv "$changesfile" "${changesfile}.binonly"
mergechanges "$schangesfile" "${changesfile}.binonly" > "$changesfile"
rm "${changesfile}.binonly" "$schangesfile"

DATESTR="$(date +%Y.%m.%d.%H%M%S)"
BUILD_PREFIX="${packagenameversion}_${arch}.${DATESTR}"

mv current "${BUILD_PREFIX}".buildlog
rm -f current-*

bundle="${BUNDLE_DIR}/${BUILD_PREFIX}".bundle.tar"
tar -chf "$bundle" *

rm -r "$BUILDDIR"
trap - 0

echo "Bundle available at $bundle ."

exit 0
